# -* coding: utf-8 -*-
import inspect
from abjad.tools import durationtools
from abjad.tools import expressiontools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Segment import Segment
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import new


class PitchClassSegment(Segment):
    r'''Pitch-class segment.

    ..  container:: example

        Initializes segment with numbered pitch-classes:

        ..  container:: example

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            ::

                >>> expression = Expression()
                >>> expression = expression.pitch_class_segment()

            ::

                >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }
            
    ..  container:: example

        Initializes segment with named pitch-classes:

        ..  container:: example

            ::

                >>> items = ['c', 'ef', 'bqs,', 'd']
                >>> segment = PitchClassSegment(
                ...     items=items,
                ...     item_class=NamedPitchClass,
                ...     )
                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            ::

                >>> expression = Expression()
                >>> expression = expression.pitch_class_segment(
                ...     item_class=NamedPitchClass,
                ...     )

            ::

                >>> segment = expression(['c', 'ef', 'bqs,', 'd'])
                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        ):
        if not items and not item_class:
            item_class = self._named_item_class
        Segment.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    @expressiontools.Signature(
        markup_expression_callback='_make___add___markup_expression',
        string_template_callback='_make___add___string_template',
        )
    def __add__(self, argument):
        r'''Adds `argument` to segment.

        ..  container:: example

            Example segments:

            ::

                >>> pitch_numbers = [-2, -1.5, 6, 7, -1.5, 7]
                >>> PitchClassSegment(items=pitch_numbers)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> J = PitchClassSegment(items=pitch_numbers)
                >>> show(J) # doctest: +SKIP

            ::

                >>> pitch_names = ['c', 'ef', 'bqs,', 'd']
                >>> PitchClassSegment(items=pitch_names)
                PitchClassSegment("c ef bqs d")

            ::

                >>> K = PitchClassSegment(items=pitch_names)
                >>> show(K) # doctest: +SKIP

        ..  container:: example

            Adds J and K:

            ..  container:: example

                ::

                    >>> J + K
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 0, 3, 11.5, 2])

                ::

                    >>> segment = J + K
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        c'8
                        ef'8
                        bqs'8
                        d'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression_J = Expression(name='J')
                    >>> expression_J = expression_J.pitch_class_segment()
                    >>> expression_K = Expression(name='K')
                    >>> expression_K = expression_K.pitch_class_segment()
                    >>> expression = expression_J + expression_K

                ::

                    >>> expression(pitch_numbers, pitch_names)
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 0, 3, 11.5, 2])

                ::

                    >>> expression.get_string()
                    'J + K'

                ::

                    >>> segment = expression(pitch_numbers, pitch_names)
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \line
                                    {
                                        \bold
                                            J
                                        +
                                        \bold
                                            K
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        c'8
                        ef'8
                        bqs'8
                        d'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Adds K and J:

            ..  container:: example

                ::

                    >>> K + J
                    PitchClassSegment("c ef bqs d bf bqf fs g bqf g")

                ::

                    >>> segment = K + J
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        c'8
                        ef'8
                        bqs'8
                        d'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression_J = Expression(name='J')
                    >>> expression_J = expression_J.pitch_class_segment()
                    >>> expression_K = Expression(name='K')
                    >>> expression_K = expression_K.pitch_class_segment()
                    >>> expression = expression_K + expression_J

                ::

                    >>> expression(pitch_names, pitch_numbers)
                    PitchClassSegment("c ef bqs d bf bqf fs g bqf g")

                ::

                    >>> expression.get_string()
                    'K + J'

                ::

                    >>> segment = expression(pitch_names, pitch_numbers)
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        c'8
                            ^ \markup {
                                \line
                                    {
                                        \bold
                                            K
                                        +
                                        \bold
                                            J
                                    }
                                }
                        ef'8
                        bqs'8
                        d'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Adds J repeatedly:

            ..  container:: example

                ::

                    >>> J + J + J
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])


                ::

                    >>> segment = J + J + J
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                    >>> expression_J = Expression(name='J')
                    >>> expression_J = expression_J.pitch_class_segment()
                    >>> expression = expression_J + expression_J + expression_J

                ::

                    >>> expression(pitch_numbers, pitch_numbers, pitch_numbers)
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7, 10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> expression.get_string()
                    'J + J + J'

                ::

                    >>> list_ = [pitch_numbers, pitch_numbers, pitch_numbers]
                    >>> segment = expression(*list_)
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \line
                                    {
                                        \line
                                            {
                                                \bold
                                                    J
                                                +
                                                \bold
                                                    J
                                            }
                                        +
                                        \bold
                                            J
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Adds transformed segments:

            ..  container:: example

                ::

                    >>> J.rotate(n=1) + K.rotate(n=2)
                    PitchClassSegment([7, 10, 10.5, 6, 7, 10.5, 11.5, 2, 0, 3])

                ::

                    >>> segment = J.rotate(n=1) + K.rotate(n=2)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        bqs'8
                        d'8
                        c'8
                        ef'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression_J = Expression(name='J')
                    >>> expression_J = expression_J.pitch_class_segment()
                    >>> expression_J = expression_J.rotate(n=1)
                    >>> expression_K = Expression(name='K')
                    >>> expression_K = expression_K.pitch_class_segment()
                    >>> expression_K = expression_K.rotate(n=2)
                    >>> expression = expression_J + expression_K

                ::

                    >>> expression(pitch_numbers, pitch_names)
                    PitchClassSegment([7, 10, 10.5, 6, 7, 10.5, 11.5, 2, 0, 3])

                ::

                    >>> expression.get_string()
                    'r1(J) + r2(K)'

                ::
                    
                    >>> segment = expression(pitch_numbers, pitch_names)
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                            ^ \markup {
                                \line
                                    {
                                        \concat
                                            {
                                                r
                                                \sub
                                                    1
                                                \bold
                                                    J
                                            }
                                        +
                                        \concat
                                            {
                                                r
                                                \sub
                                                    2
                                                \bold
                                                    K
                                            }
                                    }
                                }
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        bqs'8
                        d'8
                        c'8
                        ef'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Reverses result:

            ..  container:: example

                ::

                    >>> segment = J.rotate(n=1) + K.rotate(n=2)
                    >>> segment.retrograde()
                    PitchClassSegment([3, 0, 2, 11.5, 10.5, 7, 6, 10.5, 10, 7])

                ::

                    >>> segment = segment.retrograde()
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        ef'8
                        c'8
                        d'8
                        bqs'8
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression_J = Expression(name='J')
                    >>> expression_J = expression_J.pitch_class_segment()
                    >>> expression_J = expression_J.rotate(n=1)
                    >>> expression_K = Expression(name='K')
                    >>> expression_K = expression_K.pitch_class_segment()
                    >>> expression_K = expression_K.rotate(n=2)
                    >>> expression = expression_J + expression_K
                    >>> expression = expression.retrograde()

                ::

                    >>> expression(pitch_numbers, pitch_names)
                    PitchClassSegment([3, 0, 2, 11.5, 10.5, 7, 6, 10.5, 10, 7])

                ::

                    >>> expression.get_string()
                    'R(r1(J) + r2(K))'

                ::
                    
                    >>> segment = expression(pitch_numbers, pitch_names)
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        ef'8
                            ^ \markup {
                                \concat
                                    {
                                        R
                                        \concat
                                            {
                                                (
                                                \line
                                                    {
                                                        \concat
                                                            {
                                                                r
                                                                \sub
                                                                    1
                                                                \bold
                                                                    J
                                                            }
                                                        +
                                                        \concat
                                                            {
                                                                r
                                                                \sub
                                                                    2
                                                                \bold
                                                                    K
                                                            }
                                                    }
                                                )
                                            }
                                    }
                                }
                        c'8
                        d'8
                        bqs'8
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example expression

            Establishes equivalence:

            ::

                >>> expression_J = Expression(name='J')
                >>> expression_J = expression_J.pitch_class_segment()
                >>> expression_J = expression_J.rotate(n=1)
                >>> expression_K = Expression(name='K')
                >>> expression_K = expression_K.pitch_class_segment()
                >>> expression_K = expression_K.rotate(n=2)
                >>> expression = expression_J + expression_K
                >>> expression = expression.establish_equivalence(name='Q')

            ::

                >>> expression(pitch_numbers, pitch_names)
                PitchClassSegment([7, 10, 10.5, 6, 7, 10.5, 11.5, 2, 0, 3])

            ::

                >>> expression.get_string()
                'Q = r1(J) + r2(K)'

            ::

                >>> segment = expression(pitch_numbers, pitch_names)
                >>> markup = expression.get_markup()
                >>> show(segment, figure_name=markup) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__(
                ...     figure_name=markup,
                ...     )
                >>> f(lilypond_file[Voice])
                \new Voice {
                    g'8
                        ^ \markup {
                            \line
                                {
                                    \bold
                                        Q
                                    =
                                    \line
                                        {
                                            \concat
                                                {
                                                    r
                                                    \sub
                                                        1
                                                    \bold
                                                        J
                                                }
                                            +
                                            \concat
                                                {
                                                    r
                                                    \sub
                                                        2
                                                    \bold
                                                        K
                                                }
                                        }
                                }
                            }
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    bqs'8
                    d'8
                    c'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            Transforms equivalence:

            ::

                >>> expression = expression.transpose(n=1)

            ::

                >>> expression(pitch_numbers, pitch_names)
                PitchClassSegment([8, 11, 11.5, 7, 8, 11.5, 0.5, 3, 1, 4])

            ::

                >>> expression.get_string()
                'T1(Q)'

            ::

                >>> segment = expression(pitch_numbers, pitch_names)
                >>> markup = expression.get_markup()
                >>> show(segment, figure_name=markup) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__(
                ...     figure_name=markup,
                ...     )
                >>> f(lilypond_file[Voice])
                \new Voice {
                    af'8
                        ^ \markup {
                            \concat
                                {
                                    T
                                    \sub
                                        1
                                    \bold
                                        Q
                                }
                            }
                    b'8
                    bqs'8
                    g'8
                    af'8
                    bqs'8
                    cqs'8
                    ef'8
                    cs'8
                    e'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        Returns new segment.
        '''
        if self._expression:
            return self._update_expresion(inspect.currentframe())
        argument = type(self)(items=argument)
        items = self.items + argument.items
        return type(self)(items=items)

    def __format__(self, format_specification=''):
        r'''Formats segment.

        ..  container:: example

            With numbered pitch-classes:

            ::

                >>> pitch_numbers = [-2, -1.5, 6, 7, -1.5, 7]
                >>> J = PitchClassSegment(items=pitch_numbers)

            ::

                >>> f(J)
                pitchtools.PitchClassSegment(
                    (
                        pitchtools.NumberedPitchClass(10),
                        pitchtools.NumberedPitchClass(10.5),
                        pitchtools.NumberedPitchClass(6),
                        pitchtools.NumberedPitchClass(7),
                        pitchtools.NumberedPitchClass(10.5),
                        pitchtools.NumberedPitchClass(7),
                        ),
                    item_class=pitchtools.NumberedPitchClass,
                    )

            ::

                >>> show(J) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = J.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            With named pitch-classes:

            ::

                >>> pitch_names = ['c', 'ef', 'bqs,', 'd']
                >>> K = PitchClassSegment(
                ...     items=pitch_names,
                ...     item_class=NamedPitchClass,
                ...     )

            ::

                >>> f(K)
                pitchtools.PitchClassSegment(
                    (
                        pitchtools.NamedPitchClass('c'),
                        pitchtools.NamedPitchClass('ef'),
                        pitchtools.NamedPitchClass('bqs'),
                        pitchtools.NamedPitchClass('d'),
                        ),
                    item_class=pitchtools.NamedPitchClass,
                    )

            ::

                >>> show(K) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = K.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        Returns string.
        '''
        superclass = super(PitchClassSegment, self)
        return superclass.__format__(format_specification=format_specification)

    @expressiontools.Signature(
        markup_expression_callback='_make___getitem___markup_expression',
        string_template_callback='_make___getitem___string_template',
        )
    def __getitem__(self, argument):
        r'''Gets `argument` from segment.

        ..  container:: example

            Example segment:

            ::

                >>> pitch_numbers = [-2, -1.5, 6, 7, -1.5, 7]
                >>> PitchClassSegment(items=pitch_numbers)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> J = PitchClassSegment(items=pitch_numbers)
                >>> show(J) # doctest: +SKIP

        ..  container:: example

            Gets item at nonnegative index:

            ..  container:: example

                ::

                    >>> J[0]
                    NumberedPitchClass(10)

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression[0]

                ::

                    >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    NumberedPitchClass(10)

                ::

                    >>> expression.get_string()
                    'J[0]'

                ::

                    >>> markup = expression.get_markup()
                    >>> show(markup) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup)
                    \markup {
                        \concat
                            {
                                \bold
                                    J
                                \sub
                                    0
                            }
                        }

        ..  container:: example

            Gets item at negative index:

            ..  container:: example

                ::

                    >>> J[-1]
                    NumberedPitchClass(7)

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression[-1]

                ::

                    >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    NumberedPitchClass(7)

                ::

                    >>> expression.get_string()
                    'J[-1]'

                ::

                    >>> markup = expression.get_markup()
                    >>> show(markup) # doctest: +SKIP

                ..  doctest::

                    >>> f(markup)
                    \markup {
                        \concat
                            {
                                \bold
                                    J
                                \sub
                                    -1
                            }
                        }

        ..  container:: example

            Gets slice:

            ..  container:: example

                ::

                    >>> J[:4]
                    PitchClassSegment([10, 10.5, 6, 7])

                ::

                    >>> segment = J[:4]
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression[:4]

                ::
                
                    >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10, 10.5, 6, 7])

                ::

                    >>> expression.get_string()
                    'J[:4]'

                ::

                    >>> segment = expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \concat
                                    {
                                        \bold
                                            J
                                        \sub
                                            [:4]
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Gets retrograde of slice:

            ..  container:: example

                ::

                    >>> J[:4].retrograde()
                    PitchClassSegment([7, 6, 10.5, 10])

                ::

                    >>> segment = J[:4].retrograde()
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression[:4]
                    >>> expression = expression.retrograde()

                ::

                    >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([7, 6, 10.5, 10])

                ::

                    >>> expression.get_string()
                    'R(J[:4])'

                ::

                    >>> segment = expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                            ^ \markup {
                                \concat
                                    {
                                        R
                                        \concat
                                            {
                                                \bold
                                                    J
                                                \sub
                                                    [:4]
                                            }
                                    }
                                }
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Gets slice of retrograde:

            ..  container:: example

                ::

                    >>> J.retrograde()[:4]
                    PitchClassSegment([7, 10.5, 7, 6])

                ::

                    >>> segment = J.retrograde()[:4]
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                        bqf'8
                        g'8
                        fs'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.retrograde()
                    >>> expression = expression[:4]

                ::

                    >>> expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([7, 10.5, 7, 6])

                ::

                    >>> expression.get_string()
                    'R(J)[:4]'

                ::

                    >>> segment = expression(items=[-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                            ^ \markup {
                                \concat
                                    {
                                        \concat
                                            {
                                                (
                                                \concat
                                                    {
                                                        R
                                                        \bold
                                                            J
                                                    }
                                                )
                                            }
                                        \sub
                                            [:4]
                                    }
                                }
                        bqf'8
                        g'8
                        fs'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class or pitch-class segment:

            ::

                >>> isinstance(segment, PitchClassSegment)
                True

        '''
        if self._expression:
            return self._update_expression(
                inspect.currentframe(),
                precedence=100,
                )
        superclass = super(PitchClassSegment, self)
        return superclass.__getitem__(argument)

    def __illustrate__(self, expression_markup_direction=Up, **keywords):
        r'''Illustrates segment.

        ..  container:: example

            Illustrates numbered segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Illustrates named segment:

            ::

                >>> items = ['c', 'ef', 'bqs,', 'd']
                >>> segment = PitchClassSegment(
                ...     items=items,
                ...     item_class=NumberedPitchClass,
                ...     )
                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    c'8
                    ef'8
                    bqs'8
                    d'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns LilyPond file:

            ::

                >>> prototype = lilypondfiletools.LilyPondFile
                >>> isinstance(segment.__illustrate__(), prototype)
                True

        '''
        superclass = super(PitchClassSegment, self)
        return superclass.__illustrate__(
            expression_markup_direction=expression_markup_direction,
            **keywords
            )

    def __repr__(self):
        r'''Gets interpreter representation.

        ..  container:: example

            Interpreter representation:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> PitchClassSegment(items=items)
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

        Returns string.
        '''
        import abjad
        if self.item_class is abjad.pitchtools.NamedPitchClass:
            contents = ' '.join([str(_) for _ in self])
            contents = '"' + contents + '"'
        else:
            contents = ', '.join([str(_) for _ in self])
            contents = '[' + contents + ']'
        return '{}({})'.format(type(self).__name__, contents)

    def __str__(self):
        r'''Gets string representation of pitch-class segment.

        ..  container::

            Gets string represenation of numbered pitch class:

            ::

                >>> segment = PitchClassSegment([-2, -1.5, 6, 7, -1.5, 7])
                >>> str(segment)
                'PC<10, 10.5, 6, 7, 10.5, 7>'

        ..  container::

            Gets string represenation of named pitch class:

            ::

                >>> segment = PitchClassSegment(
                ...     items=[-2, -1.5, 6, 7, -1.5, 7],
                ...     item_class=NamedPitchClass,
                ...     )
                >>> str(segment)
                'PC<bf bqf fs g bqf g>'

        Returns string.
        '''
        import abjad
        items = [str(_) for _ in self]
        separator = ' '
        if self.item_class is abjad.NumberedPitchClass:
            separator = ', '
        return 'PC<{}>'.format(separator.join(items))

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.PitchClass

    ### PRIVATE METHODS ###

    def _get_padded_string(self, width=2):
        superclass = super(PitchClassSegment, self)
        string = superclass._get_padded_string(width=width)
        return 'PC<' + string[1:-1] + '>'

    def _is_equivalent_under_transposition(self, argument):
        r'''Is true when `argument` is equivalent to segment under
        transposition.
        
        Otherwise False.

        Returns true or false.
        '''
        import abjad
        if not isinstance(argument, type(self)):
            return False
        if not len(self) == len(argument):
            return False
        difference = -(abjad.NamedPitch(argument[0], 4) -
            abjad.NamedPitch(self[0], 4))
        new_pitch_classes = (x + difference for x in self)
        new_pitch_classes = new(self, items=new_pitch_classes)
        return argument == new_pitch_classes

    @staticmethod
    def _make_rotate_method_name(n=0, stravinsky=False):
        if stravinsky:
            return 'rs'
        return 'r'

    def _update_expression(self, frame, precedence=None):
        callback = expressiontools.Expression._frame_to_callback(
            frame,
            precedence=precedence,
            )
        return self._expression.append_callback(callback)

    ### PUBLIC PROPERTIES ###

    @property
    def item_class(self):
        r'''Gets item class of segment.

        ..  container:: example

            Gets item class of numbered segment:
        
            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

            ::

                >>> segment.item_class
                <class 'abjad.tools.pitchtools.NumberedPitchClass.NumberedPitchClass'>

        ..  container:: example


            Gets item class of named segment:

            ::

                >>> items = ['c', 'ef', 'bqs,', 'd']
                >>> segment = PitchClassSegment(
                ...     items=items,
                ...     item_class=NamedPitchClass,
                ...     )
                >>> show(segment) # doctest: +SKIP

            ::
                
                >>> segment.item_class
                <class 'abjad.tools.pitchtools.NamedPitchClass.NamedPitchClass'>

        ..  container:: example

            Returns class:

            ::

                >>> type(segment.item_class)
                <class 'abc.ABCMeta'>

        '''
        superclass = super(PitchClassSegment, self)
        return superclass.item_class

    @property
    def items(self):
        r'''Gets items in segment.

        ..  container:: example

            ..  container:: example

                Initializes items positionally:

                ::

                    >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                    >>> segment = PitchClassSegment(items)
                    >>> for item in segment.items:
                    ...     item
                    ...
                    NumberedPitchClass(10)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(6)
                    NumberedPitchClass(7)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(7)

                Initializes items from keyword:

                ::

                    >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                    >>> segment = PitchClassSegment(items=items)
                    >>> for item in segment.items:
                    ...     item
                    NumberedPitchClass(10)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(6)
                    NumberedPitchClass(7)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(7)

            ..  container:: example expression

                Initializes items positionally:

                ::

                    >>> expression = Expression()
                    >>> expression = expression.pitch_class_segment()

                ::

                    >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                    >>> segment = expression(items)
                    >>> for item in segment.items:
                    ...     item
                    NumberedPitchClass(10)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(6)
                    NumberedPitchClass(7)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(7)

                Initializes items from keyword:

                ::

                    >>> expression = Expression()
                    >>> expression = expression.pitch_class_segment()

                ::

                    >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                    >>> segment = expression(items=items)
                    >>> for item in segment.items:
                    ...     item
                    NumberedPitchClass(10)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(6)
                    NumberedPitchClass(7)
                    NumberedPitchClass(10.5)
                    NumberedPitchClass(7)

        ..  container:: example

            Returns list:

            ::

                >>> isinstance(segment.items, list)
                True

        '''
        superclass = super(PitchClassSegment, self)
        return superclass.items

    ### PUBLIC METHODS ###

    def count(self, item):
        r'''Counts `item` in segment.

        ..  container:: example

            Example segment:
        
            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

        ..  container:: example

            Counts existing item in segment:

            ::
                
                >>> segment.count(-1.5)
                2

        ..  container:: example

            Counts nonexisting item in segment:

            ::
                
                >>> segment.count('text')
                0

        ..  container:: example

            Returns nonnegative integer:

            ::

                >>> isinstance(segment.count('text'), int)
                True

        '''
        superclass = super(PitchClassSegment, self)
        return superclass.count(item)

    @classmethod
    def from_selection(class_, selection, item_class=None):
        r'''Initializes segment from `selection`.

        ..  container:: example

            itializes from selection:

            ::

                >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
                >>> staff_2 = Staff("c4. r8 g2")
                >>> staff_group = StaffGroup([staff_1, staff_2])
                >>> show(staff_group) # doctest: +SKIP

            ::

                >>> selection = select((staff_1, staff_2))
                >>> segment = PitchClassSegment.from_selection(selection)
                >>> show(segment) # doctest: +SKIP

        ..  container:: example

            Returns pitch-class segment:

            ::

                >>> segment
                PitchClassSegment("c d fs a b c g")

        '''
        import abjad
        pitch_segment = abjad.pitchtools.PitchSegment.from_selection(selection)
        return class_(
            items=pitch_segment,
            item_class=item_class,
            )

    def has_duplicates(self):
        r'''Is true when segment contains duplicate items. Otherwise false.

        ..  container:: example

            Has duplicates:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

            ::

                >>> segment.has_duplicates()
                True

        ..  container:: example

            Has no duplicates:

            ::

                >>> items = "c d e f g a b"
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

            ::

                >>> segment.has_duplicates()
                False

        Returns true or false.
        '''
        import abjad
        return len(abjad.pitchtools.PitchClassSet(self)) < len(self)

    def index(self, item):
        r'''Gets index of `item` in segment.

        ..  container:: example

            Example segment:
        
            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

        ..  container:: example

            Gets index of first item in segment:
        
            ::

                >>> segment.index(-2)
                0

        ..  container:: example

            Gets index of second item in segment:

            ::
                
                >>> segment.index(-1.5)
                1

        ..  container:: example

            Returns nonnegative integer:

            ::
                
                >>> isinstance(segment.index(-1.5), int)
                True

        '''
        superclass = super(PitchClassSegment, self)
        return superclass.index(item)

    @expressiontools.Signature(
        is_operator=True,
        method_name='I',
        subscript='axis',
        )
    def invert(self, axis=None):
        r'''Inverts segment.

        ..  container:: example

            Example segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> J = PitchClassSegment(items=items)
                >>> J
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(J) # doctest: +SKIP

        ..  container:: example

            Inverts segment:

            ..  container:: example

                ::

                    >>> J.invert()
                    PitchClassSegment([2, 1.5, 6, 5, 1.5, 5])

                ::

                    >>> segment = J.invert()
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        d'8
                        dqf'8
                        fs'8
                        f'8
                        dqf'8
                        f'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.invert()

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([2, 1.5, 6, 5, 1.5, 5])

                ::

                    >>> expression.get_string()
                    'I(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        d'8
                            ^ \markup {
                                \concat
                                    {
                                        I
                                        \bold
                                            J
                                    }
                                }
                        dqf'8
                        fs'8
                        f'8
                        dqf'8
                        f'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Inverts inversion of segment:

            ..  container:: example

                ::

                    >>> J.invert().invert()
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> segment = J.invert().invert()
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

                ::

                    >>> segment == J
                    True

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.invert()
                    >>> expression = expression.invert()

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> expression.get_string()
                    'I(I(J))'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \concat
                                    {
                                        I
                                        \concat
                                            {
                                                I
                                                \bold
                                                    J
                                            }
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class segment:

            ::

                >>> isinstance(segment, PitchClassSegment)
                True

        '''
        if self._expression:
            return self._update_expression(inspect.currentframe())
        items = [_.invert(axis=axis) for _ in self]
        return type(self)(items=items)

    def make_notes(self, n=None, written_duration=None):
        r'''Makes first `n` notes in segment.

        ..  container:: example

            Example segment:

            ::

                >>> items = [2, 4.5, 6, 11, 4.5, 10]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

        ..  container:: example

            Makes eighth notes:

            ::

                >>> notes = segment.make_notes()
                >>> staff = Staff(notes)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    d'8
                    eqs'8
                    fs'8
                    b'8
                    eqs'8
                    bf'8
                }

        ..  container:: example

            Makes notes with nonassignable durations:

            ::

                >>> notes = segment.make_notes(4, Duration(5, 16))
                >>> staff = Staff(notes)
                >>> time_signature = TimeSignature((5, 4))
                >>> attach(time_signature, staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \time 5/4
                    d'4 ~
                    d'16
                    eqs'4 ~
                    eqs'16
                    fs'4 ~
                    fs'16
                    b'4 ~
                    b'16
                }

        Interprets none-valued `n` equal to length of segment.

        Interprets none-valued `written_duration` equal to 1/8.

        ..  container:: example

            Returns selection:

            ::

                >>> isinstance(segment.make_notes(), selectiontools.Selection)
                True

        '''
        import abjad
        n = n or len(self)
        written_duration = written_duration or abjad.Duration(1, 8)
        result = abjad.scoretools.make_notes([0] * n, [written_duration])
        logical_ties = abjad.iterate(result).by_logical_tie()
        for i, logical_tie in enumerate(logical_ties):
            pitch_class = abjad.pitchtools.NamedPitchClass(self[i % len(self)])
            pitch = abjad.NamedPitch(pitch_class, 4)
            for note in logical_tie:
                note.written_pitch = pitch
        return result

    @expressiontools.Signature(
        is_operator=True,
        method_name='M',
        subscript='n',
        )
    def multiply(self, n=1):
        r'''Multiplies pitch-classes in segment by `n`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> J = PitchClassSegment(items=items)
                >>> J
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(J) # doctest: +SKIP

        ..  container:: example

            Multiplies pitch-classes in segment by 1:

            ..  container:: example

                ::

                    >>> J.multiply(n=1)
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> segment = J.multiply(n=1)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.multiply(n=1)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> expression.get_string()
                    'M1(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \concat
                                    {
                                        M
                                        \sub
                                            1
                                        \bold
                                            J
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Multiplies pitch-classes in segment by 5:

            ..  container:: example

                ::

                    >>> J.multiply(n=5)
                    PitchClassSegment([2, 4.5, 6, 11, 4.5, 11])

                ::

                    >>> segment = J.multiply(n=5)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        d'8
                        eqs'8
                        fs'8
                        b'8
                        eqs'8
                        b'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.multiply(n=5)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([2, 4.5, 6, 11, 4.5, 11])

                ::

                    >>> expression.get_string()
                    'M5(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        d'8
                            ^ \markup {
                                \concat
                                    {
                                        M
                                        \sub
                                            5
                                        \bold
                                            J
                                    }
                                }
                        eqs'8
                        fs'8
                        b'8
                        eqs'8
                        b'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Multiplies pitch-classes in segment by 7:

            ..  container:: example

                ::

                    >>> J.multiply(n=7)
                    PitchClassSegment([10, 1.5, 6, 1, 1.5, 1])

                ::

                    >>> segment = J.multiply(n=7)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        dqf'8
                        fs'8
                        cs'8
                        dqf'8
                        cs'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.multiply(n=7)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10, 1.5, 6, 1, 1.5, 1])

                ::

                    >>> expression.get_string()
                    'M7(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \concat
                                    {
                                        M
                                        \sub
                                            7
                                        \bold
                                            J
                                    }
                                }
                        dqf'8
                        fs'8
                        cs'8
                        dqf'8
                        cs'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Multiplies pitch-classes in segment by 11:

            ..  container:: example

                ::

                    >>> segment = J.multiply(n=11)
                    >>> segment
                    PitchClassSegment([2, 7.5, 6, 5, 7.5, 5])

                ::

                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        d'8
                        gqs'8
                        fs'8
                        f'8
                        gqs'8
                        f'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.multiply(n=11)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([2, 7.5, 6, 5, 7.5, 5])

                ::
                
                    >>> expression.get_string()
                    'M11(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        d'8
                            ^ \markup {
                                \concat
                                    {
                                        M
                                        \sub
                                            11
                                        \bold
                                            J
                                    }
                                }
                        gqs'8
                        fs'8
                        f'8
                        gqs'8
                        f'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class segment:

            ::

                >>> isinstance(segment, PitchClassSegment)
                True

        '''
        import abjad
        if self._expression:
            return self._update_expression(inspect.currentframe())
        items = [abjad.pitchtools.NumberedPitchClass(_) for _ in self]
        items = [_.multiply(n) for _ in items]
        return type(self)(items=items)

    @expressiontools.Signature()
    def permute(self, row=None):
        r'''Permutes segment by twelve-tone `row`.

        ..  container:: example

            ::

                >>> PitchClassSegment([-2, -1, 6, 7, -1, 7])
                PitchClassSegment([10, 11, 6, 7, 11, 7])

            ::

                >>> segment = PitchClassSegment([-2, -1, 6, 7, -1, 7])
                >>> show(segment) # doctest: +SKIP

            ..  doctest:

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    b'8
                    fs'8
                    g'8
                    b'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            ::

                >>> segment.permute([10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]) 
                PitchClassSegment([4, 11, 5, 3, 11, 3])

            ::

                >>> segment = segment.permute([10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]) 
                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    e'8
                    b'8
                    f'8
                    ef'8
                    b'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example expression

            ::

                >>> expression = Expression(name='J')
                >>> expression = expression.pitch_class_segment()
                >>> row = [10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11]
                >>> expression = expression.permute(row)

            ::

                >>> expression([-2, -1, 6, 7, -1, 7])
                PitchClassSegment([4, 11, 5, 3, 11, 3])

            ::

                >>> expression.get_string()
                'permute(J, row=[10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])'

            ::

                >>> segment = expression([-2, -1, 6, 7, -1, 7])
                >>> markup = expression.get_markup()
                >>> show(segment, figure_name=markup) # doctest: +SKIP

            ..  doctest:

                >>> lilypond_file = segment.__illustrate__(
                ...     figure_name=markup,
                ...     )
                >>> f(lilypond_file[Voice])
                \new Voice {
                    e'8
                        ^ \markup {
                            \concat
                                {
                                    permute(
                                    \bold
                                        J
                                    ", row=[10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11])"
                                }
                            }
                    b'8
                    f'8
                    ef'8
                    b'8
                    ef'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        Returns new segment.
        '''
        from abjad.tools import pitchtools
        if self._expression:
            return self._update_expression(inspect.currentframe())
        row = pitchtools.TwelveToneRow(items=row)
        items = row(self)
        return type(self)(items=items)

    @expressiontools.Signature(is_operator=True, method_name='R')
    def retrograde(self):
        r'''Gets retrograde of segment.

        ..  container:: example

            Example segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> J = PitchClassSegment(items=items)
                >>> J
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(J) # doctest: +SKIP

        ..  container:: example

            Gets retrograde of segment:

            ..  container:: example

                ::

                    >>> segment = J.retrograde()
                    >>> segment
                    PitchClassSegment([7, 10.5, 7, 6, 10.5, 10])

                ::

                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.retrograde()

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([7, 10.5, 7, 6, 10.5, 10])

                ::

                    >>> expression.get_string()
                    'R(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                            ^ \markup {
                                \concat
                                    {
                                        R
                                        \bold
                                            J
                                    }
                                }
                        bqf'8
                        g'8
                        fs'8
                        bqf'8
                        bf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Gets retrograde of retrograde of segment:

            ..  container:: example

                ::

                    >>> segment = J.retrograde().retrograde()
                    >>> segment
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

                ::

                    >>> segment == J
                    True

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.retrograde()
                    >>> expression = expression.retrograde()

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> expression.get_string()
                    'R(R(J))'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \concat
                                    {
                                        R
                                        \concat
                                            {
                                                R
                                                \bold
                                                    J
                                            }
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

                ::

                    >>> segment == J
                    True

        ..  container:: example

            Returns pitch-class segment:

            ::

                >>> isinstance(segment, PitchClassSegment)
                True

        '''
        if self._expression:
            return self._update_expression(inspect.currentframe())
        return type(self)(items=reversed(self))

    @expressiontools.Signature( 
        is_operator=True,
        method_name_callback='_make_rotate_method_name',
        subscript='n',
        )
    def rotate(self, n=0, stravinsky=False):
        r'''Rotates segment by index `n`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> J = PitchClassSegment(items=items)
                >>> J
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(J) # doctest: +SKIP

        ..  container:: example

            Rotates segment to the right:

            ..  container:: example

                ::

                    >>> J.rotate(n=1)
                    PitchClassSegment([7, 10, 10.5, 6, 7, 10.5])

                ::

                    >>> segment = J.rotate(n=1)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.rotate(n=1)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([7, 10, 10.5, 6, 7, 10.5])

                ::

                    >>> expression.get_string()
                    'r1(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        g'8
                            ^ \markup {
                                \concat
                                    {
                                        r
                                        \sub
                                            1
                                        \bold
                                            J
                                    }
                                }
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Rotates segment to the left:

            ..  container:: example

                ::

                    >>> J.rotate(n=-1)
                    PitchClassSegment([10.5, 6, 7, 10.5, 7, 10])

                ::

                    >>> segment = J.rotate(n=-1)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.rotate(n=-1)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10.5, 6, 7, 10.5, 7, 10])

                ::
                    
                    >>> expression.get_string()
                    'r-1(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bqf'8
                            ^ \markup {
                                \concat
                                    {
                                        r
                                        \sub
                                            -1
                                        \bold
                                            J
                                    }
                                }
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        bf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Rotates segment by zero:
            
            ..  container:: example

                ::

                    >>> J.rotate(n=0)
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> segment = J.rotate(n=0)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

                ::

                    >>> segment == J
                    True

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.rotate(n=0)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> expression.get_string()
                    'r0(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \concat
                                    {
                                        r
                                        \sub
                                            0
                                        \bold
                                            J
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

                ::

                    >>> segment == J
                    True

        ..  container:: example

            Stravinsky-style rotation back-transposes segment to
            begin at zero:

            ..  container:: example

                ::

                    >>> J.rotate(n=1, stravinsky=True)
                    PitchClassSegment([0, 3, 3.5, 11, 0, 3.5])

                ::

                    >>> segment = J.rotate(n=1, stravinsky=True)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        c'8
                        ef'8
                        eqf'8
                        b'8
                        c'8
                        eqf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.rotate(n=1, stravinsky=True)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([0, 3, 3.5, 11, 0, 3.5])

                ::

                    >>> expression.get_string()
                    'rs1(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        c'8
                            ^ \markup {
                                \concat
                                    {
                                        rs
                                        \sub
                                            1
                                        \bold
                                            J
                                    }
                                }
                        ef'8
                        eqf'8
                        b'8
                        c'8
                        eqf'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Returns pitch-class segment:

            ::

                >>> isinstance(segment, PitchClassSegment)
                True

        '''
        if self._expression:
            return self._update_expression(inspect.currentframe())
        original_n = n
        items = sequencetools.Sequence(self._collection).rotate(n=n)
        if stravinsky:
            n = 0 - float(items[0])
            segment = new(self, items=items)
            segment = segment.transpose(n=n)
            items = segment.items[:]
        return type(self)(items=items)

    def to_pitch_classes(self):
        r'''Changes to pitch-classes.

        ..  container:: example

            To numbered pitch-class segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> segment
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            ::

                >>> segment = segment.to_pitch_classes()
                >>> segment
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            To named pitch-class segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(
                ...     items=items,
                ...     item_class=NamedPitchClass,
                ...     )
                >>> segment
                PitchClassSegment("bf bqf fs g bqf g")

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            ::

                >>> segment = segment.to_pitch_classes()
                >>> segment
                PitchClassSegment("bf bqf fs g bqf g")

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        Returns new segment.
        '''
        import abjad
        return abjad.new(self)

    def to_pitches(self):
        r'''Changes to pitches.

        ..  container:: example

            To numbered pitch segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(items=items)
                >>> segment
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            ::

                >>> segment = segment.to_pitches()
                >>> segment
                PitchSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[StaffGroup])
                \new PianoStaff <<
                    \context Staff = "treble" {
                        \clef "treble"
                        bf'1 * 1/8
                        bqf'1 * 1/8
                        fs'1 * 1/8
                        g'1 * 1/8
                        bqf'1 * 1/8
                        g'1 * 1/8
                    }
                    \context Staff = "bass" {
                        \clef "bass"
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                    }
                >>

        ..  container:: example

            To named pitch segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> segment = PitchClassSegment(
                ...     items=items,
                ...     item_class=NamedPitchClass,
                ...     )
                >>> segment
                PitchClassSegment("bf bqf fs g bqf g")

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    bf'8
                    bqf'8
                    fs'8
                    g'8
                    bqf'8
                    g'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

            ::

                >>> segment = segment.to_pitches()
                >>> segment
                PitchSegment("bf' bqf' fs' g' bqf' g'")

            ::

                >>> show(segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment.__illustrate__()
                >>> f(lilypond_file[StaffGroup])
                \new PianoStaff <<
                    \context Staff = "treble" {
                        \clef "treble"
                        bf'1 * 1/8
                        bqf'1 * 1/8
                        fs'1 * 1/8
                        g'1 * 1/8
                        bqf'1 * 1/8
                        g'1 * 1/8
                    }
                    \context Staff = "bass" {
                        \clef "bass"
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                        r1 * 1/8
                    }
                >>

        Returns new segment.
        '''
        import abjad
        class_ = abjad.pitchtools.Pitch
        item_class = class_._to_pitch_item_class(self.item_class)
        return abjad.PitchSegment(items=self.items, item_class=item_class)

    @expressiontools.Signature(
        is_operator=True,
        method_name='T',
        subscript='n',
        )
    def transpose(self, n=0):
        r'''Transposes segment by index `n`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [-2, -1.5, 6, 7, -1.5, 7]
                >>> J = PitchClassSegment(items=items)
                >>> J
                PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

            ::

                >>> show(J) # doctest: +SKIP

        ..  container:: example

            Transposes segment by positive index:

            ..  container:: example

                ::

                    >>> J.transpose(n=13)
                    PitchClassSegment([11, 11.5, 7, 8, 11.5, 8])

                ::

                    >>> segment = J.transpose(n=13)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        b'8
                        bqs'8
                        g'8
                        af'8
                        bqs'8
                        af'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.transpose(n=13)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([11, 11.5, 7, 8, 11.5, 8])

                ::

                    >>> expression.get_string()
                    'T13(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        b'8
                            ^ \markup {
                                \concat
                                    {
                                        T
                                        \sub
                                            13
                                        \bold
                                            J
                                    }
                                }
                        bqs'8
                        g'8
                        af'8
                        bqs'8
                        af'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Transposes segment by negative index:

            ..  container:: example

                ::

                    >>> J.transpose(n=-13)
                    PitchClassSegment([9, 9.5, 5, 6, 9.5, 6])

                ::

                    >>> segment = J.transpose(n=-13)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        a'8
                        aqs'8
                        f'8
                        fs'8
                        aqs'8
                        fs'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.transpose(n=-13)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([9, 9.5, 5, 6, 9.5, 6])

                ::

                    >>> expression.get_string()
                    'T-13(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        a'8
                            ^ \markup {
                                \concat
                                    {
                                        T
                                        \sub
                                            -13
                                        \bold
                                            J
                                    }
                                }
                        aqs'8
                        f'8
                        fs'8
                        aqs'8
                        fs'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

        ..  container:: example

            Transposes segment by zero index:

            ..  container:: example

                ::

                    >>> J.transpose(n=0)
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> segment = J.transpose(n=0)
                    >>> show(segment) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__()
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

                ::

                    >>> segment == J
                    True

            ..  container:: example expression

                ::

                    >>> expression = Expression(name='J')
                    >>> expression = expression.pitch_class_segment()
                    >>> expression = expression.transpose(n=0)

                ::

                    >>> expression([-2, -1.5, 6, 7, -1.5, 7])
                    PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

                ::

                    >>> expression.get_string()
                    'T0(J)'

                ::

                    >>> segment = expression([-2, -1.5, 6, 7, -1.5, 7])
                    >>> markup = expression.get_markup()
                    >>> show(segment, figure_name=markup) # doctest: +SKIP

                ..  doctest::

                    >>> lilypond_file = segment.__illustrate__(
                    ...     figure_name=markup,
                    ...     )
                    >>> f(lilypond_file[Voice])
                    \new Voice {
                        bf'8
                            ^ \markup {
                                \concat
                                    {
                                        T
                                        \sub
                                            0
                                        \bold
                                            J
                                    }
                                }
                        bqf'8
                        fs'8
                        g'8
                        bqf'8
                        g'8
                        \bar "|."
                        \override Score.BarLine.transparent = ##f
                    }

                ::

                    >>> segment == J
                    True

        ..  container:: example

            Returns pitch-class segment:

            ::

                >>> isinstance(segment, PitchClassSegment)
                True

        '''
        if self._expression:
            return self._update_expression(inspect.currentframe())
        items = [_.transpose(n=n) for _ in self]
        return type(self)(items=items)

    def voice_horizontally(self, initial_octave=4):
        r'''Voices segment with each pitch as close to the previous pitch as
        possible.

        ..  todo:: Should be implemented somewhere else.

        ..  container:: example

            Voices horizontally:

            ::

                >>> items = "c b d e f g e b a c"
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

            ::

                >>> voiced_segment = segment.voice_horizontally()
                >>> show(voiced_segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = voiced_segment.__illustrate__()
                >>> f(lilypond_file[Score])
                \new Score \with {
                    \override BarLine.stencil = ##f
                    \override BarNumber.transparent = ##t
                    \override Rest.transparent = ##t
                    \override SpanBar.stencil = ##f
                    \override TimeSignature.stencil = ##f
                } <<
                    \new PianoStaff <<
                        \context Staff = "treble" {
                            \clef "treble"
                            c'1 * 1/8
                            b1 * 1/8
                            d'1 * 1/8
                            e'1 * 1/8
                            f'1 * 1/8
                            g'1 * 1/8
                            e'1 * 1/8
                            b1 * 1/8
                            r1 * 1/8
                            c'1 * 1/8
                        }
                        \context Staff = "bass" {
                            \clef "bass"
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            a1 * 1/8
                            r1 * 1/8
                        }
                    >>
                >>

        ..  container:: example

            Returns pitch segment:

            ::

                >>> voiced_segment
                PitchSegment("c' b d' e' f' g' e' b a c'")

        '''
        import abjad
        initial_octave = abjad.pitchtools.Octave(initial_octave)
        pitches = []
        if self:
            pitch_class = abjad.pitchtools.NamedPitchClass(self[0])
            pitch = abjad.NamedPitch(pitch_class, initial_octave)
            pitches.append(pitch)
            for pitch_class in self[1:]:
                pitch_class = abjad.pitchtools.NamedPitchClass(pitch_class)
                pitch = abjad.NamedPitch(pitch_class, initial_octave)
                semitones = abs((pitch - pitches[-1]).semitones)
                while 6 < semitones:
                    if pitch < pitches[-1]:
                        pitch += 12
                    else:
                        pitch -= 12
                    semitones = abs((pitch - pitches[-1]).semitones)
                pitches.append(pitch)
        if self.item_class is abjad.pitchtools.NamedPitchClass:
            item_class = abjad.NamedPitch
        else:
            item_class = abjad.pitchtools.NumberedPitch
        return abjad.pitchtools.PitchSegment(
            items=pitches,
            item_class=item_class,
            )

    def voice_vertically(self, initial_octave=4):
        r'''Voices segment with each pitch higher than the previous.

        ..  todo:: Should be implemented somewhere else.

        ..  container:: example

            Voices vertically:

            ::

                >>> scale_degree_numbers = [1, 3, 5, 7, 9, 11, 13]
                >>> scale = tonalanalysistools.Scale('c', 'minor')
                >>> segment = PitchClassSegment((
                ...     scale.scale_degree_to_named_pitch_class(x)
                ...     for x in scale_degree_numbers))
                >>> show(segment) # doctest: +SKIP

            ::

                >>> voiced_segment = segment.voice_vertically()
                >>> show(voiced_segment) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = voiced_segment.__illustrate__()
                >>> f(lilypond_file[Score])
                \new Score \with {
                    \override BarLine.stencil = ##f
                    \override BarNumber.transparent = ##t
                    \override Rest.transparent = ##t
                    \override SpanBar.stencil = ##f
                    \override TimeSignature.stencil = ##f
                } <<
                    \new PianoStaff <<
                        \context Staff = "treble" {
                            \clef "treble"
                            c'1 * 1/8
                            ef'1 * 1/8
                            g'1 * 1/8
                            bf'1 * 1/8
                            d''1 * 1/8
                            f''1 * 1/8
                            af''1 * 1/8
                        }
                        \context Staff = "bass" {
                            \clef "bass"
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                            r1 * 1/8
                        }
                    >>
                >>

        ..  container:: example

            Returns pitch segment:

            ::

                >>> voiced_segment
                PitchSegment("c' ef' g' bf' d'' f'' af''")

        '''
        import abjad
        initial_octave = abjad.pitchtools.Octave(initial_octave)
        pitches = []
        if self:
            pitch_class = abjad.pitchtools.NamedPitchClass(self[0])
            pitch = abjad.NamedPitch(pitch_class, initial_octave)
            pitches.append(pitch)
            for pitch_class in self[1:]:
                pitch_class = abjad.pitchtools.NamedPitchClass(pitch_class)
                pitch = abjad.NamedPitch(pitch_class, initial_octave)
                while pitch < pitches[-1]:
                    pitch += 12
                pitches.append(pitch)
        if self.item_class is abjad.pitchtools.NamedPitchClass:
            item_class = abjad.NamedPitch
        else:
            item_class = abjad.pitchtools.NumberedPitch
        return abjad.pitchtools.PitchSegment(
            items=pitches,
            item_class=item_class,
            )



# -*- coding: utf-8 -*-
import pickle
from abjad import *


def test_quantizationtools_QGrid_pickle_01():
    q_grid = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1,
            children=[
                quantizationtools.QGridLeaf(
                    preprolated_duration=1,
                    q_event_proxies=[
                        quantizationtools.QEventProxy(
                            quantizationtools.SilentQEvent(100),
                            0.5,
                            ),
                        ],
                    ),
                ],
            ),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9,
                    ),
                ],
            ),
        )
    pickled = pickle.loads(pickle.dumps(q_grid))

    assert format(pickled) == format(q_grid)
    assert pickled is not q_grid
    assert pickled != q_grid, \
        systemtools.TestManager.diff(pickled, q_grid, 'Diff:')



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_tonalanalysistools_RootedChordClass_extent_to_extent_name_01():

    assert tonalanalysistools.RootedChordClass.extent_to_extent_name(5) == 'triad'
    assert tonalanalysistools.RootedChordClass.extent_to_extent_name(7) == 'seventh'
    assert tonalanalysistools.RootedChordClass.extent_to_extent_name(9) == 'ninth'
    assert tonalanalysistools.RootedChordClass.extent_to_extent_name(11) == 'eleventh'
    assert tonalanalysistools.RootedChordClass.extent_to_extent_name(13) == 'thirteenth'


def test_tonalanalysistools_RootedChordClass_extent_to_extent_name_02():

    string = 'tonalanalysistools.RootedChordClass.extent_to_extent_name(1)'
    assert pytest.raises(Exception, string)



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TupletRhythmMaker___call___01():
    r'''TupletRhythmMaker can make tuplet monads.
    '''


    tuplet_ratios = [(1,)]
    maker = rhythmmakertools.TupletRhythmMaker(tuplet_ratios=tuplet_ratios)


    divisions = [(1, 5), (1, 4), (1, 6), (7, 9)]
    tuplet_lists = maker(divisions)
    tuplets = Sequence(tuplet_lists).flatten()
    staff = Staff(tuplets)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tweak edge-height #'(0.7 . 0)
            \times 4/5 {
                c'4
            }
            {
                c'4
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'4
            }
            \tweak edge-height #'(0.7 . 0)
            \times 8/9 {
                c'2..
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Cluster___init___01():
    r'''Cluster can be empty.
    '''
    cluster = scoretools.Cluster([])
    assert not cluster.is_simultaneous
    assert len(cluster) == 0
    assert format(cluster) == stringtools.normalize(
        r'''
        \makeClusters {
        }
        '''
        )


def test_scoretools_Cluster___init___02():
    cluster = scoretools.Cluster(Note(1, (1, 4)) * 4)
    assert isinstance(cluster, scoretools.Cluster)
    assert not cluster.is_simultaneous
    assert len(cluster) == 4
    assert format(cluster) == stringtools.normalize(
        r'''
        \makeClusters {
            cs'4
            cs'4
            cs'4
            cs'4
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.scoretools.Context import Context


class Staff(Context):
    r'''A staff.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }


    ::

        >>> show(staff) # doctest: +SKIP

    Returns Staff instance.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Contexts'

    __slots__ = ()

    _default_context_name = 'Staff'

    ### INITIALIZER ###

    def __init__(
        self,
        music=None,
        context_name='Staff',
        is_simultaneous=None,
        name=None,
        ):
        Context.__init__(
            self,
            music=music,
            context_name=context_name,
            is_simultaneous=is_simultaneous,
            name=name,
            )



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os
import setuptools
import sys
from distutils.version import StrictVersion

version_file_path = os.path.join(
    os.path.dirname(__file__),
    'abjad',
    '_version.py'
    )
with open(version_file_path, 'r') as file_pointer:
    file_contents_string = file_pointer.read()
local_dict = {}
exec(file_contents_string, None, local_dict)
__version__ = local_dict['__version__']

description = 'Abjad is a Python API for Formalized Score Control.'

with open('README.rst', 'r') as file_pointer:
    long_description = file_pointer.read()

author = [
    'Trevor Bača',
    'Josiah Wolf Oberholtzer',
    'Víctor Adán',
    ]
author = ', '.join(author)

author_email = [
    'trevorbaca@gmail.com',
    'josiah.oberholtzer@gmail.com',
    'contact@victoradan.net',
    ]
author_email = ', '.join(author_email)

keywords = [
    'music composition',
    'music notation',
    'formalized score control',
    'lilypond',
    ]
keywords = ', '.join(keywords)

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Topic :: Artistic Software',
    ]

install_requires = [
    'ply',
    'six',
    ]
version = '.'.join(str(x) for x in sys.version_info[:3])
if StrictVersion(version) < StrictVersion('3.4.0'):
    install_requires.append('enum34')
    install_requires.append('pathlib2')
if StrictVersion(version) < StrictVersion('3.3.0'):
    install_requires.append('funcsigs')
    install_requires.append('mock')

extras_require = {
    'accelerated': [
        'quicktions>=1.3',
        ],
    'development': [
        'pytest>=3.0.0',
        'sphinx>=1.5.0',
        'sphinx-rtd-theme',
        'PyPDF2',
        ],
    'ipython': [
        'ipython',
        'jupyter',
        ],
    }

entry_points = {
    'console_scripts': [
        'abjad = abjad.tools.systemtools.run_abjad:run_abjad',
        'ajv = abjad.tools.commandlinetools.run_ajv:run_ajv',
        ]
    }

if __name__ == '__main__':
    setuptools.setup(
        author=author,
        author_email=author_email,
        classifiers=classifiers,
        description=description,
        entry_points=entry_points,
        extras_require=extras_require,
        include_package_data=True,
        install_requires=install_requires,
        keywords=keywords,
        license='GPL',
        long_description=long_description,
        name='Abjad',
        packages=['abjad'],
        platforms='Any',
        url='http://www.projectabjad.org',
        version=__version__,
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo___eq___01():
    r'''Tempos compare equal when durations and units per minute match.
    '''

    tempo_1 = Tempo(Duration(3, 32), 52)
    tempo_2 = Tempo(Duration(3, 32), 52)
    assert tempo_1 == tempo_2


def test_indicatortools_Tempo___eq___02():
    r'''Tempos do not compare equal when mathematically equal.
    '''

    tempo_1 = Tempo(Duration(3, 32), 52)
    tempo_2 = Tempo(Duration(6, 32), 104)
    assert not tempo_1 == tempo_2


def test_indicatortools_Tempo___eq___03():
    r'''Tempos also compare textual indications.
    '''

    tempo_1 = Tempo(Duration(3, 32), 52, 'Langsam')
    tempo_2 = Tempo(Duration(3, 32), 52, 'Langsam')
    tempo_3 = Tempo(Duration(3, 32), 52, 'Slow')
    assert tempo_1 == tempo_2

    assert not tempo_1 == tempo_3



# -*- coding: utf-8 -*-
from abjad.tools import patterntools


def sustain_first(n=1, inverted=None):
    r'''Makes sustain mask that matches the first `n` indices.

    ..  container:: example

        Sustains first division:

        ::

            >>> mask = rhythmmakertools.sustain_first()

        ::

            >>> f(mask)
            rhythmmakertools.SustainMask(
                pattern=patterntools.Pattern(
                    indices=[0],
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
            ...     denominators=[16],
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }
                {
                    \time 7/16
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }
                {
                    \time 3/8
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }
            }

    ..  container:: example

        Sustains first two divisions:

        ::

            >>> mask = rhythmmakertools.sustain_first(n=2)

        ::

            >>> f(mask)
            rhythmmakertools.SustainMask(
                pattern=patterntools.Pattern(
                    indices=[0, 1],
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.EvenDivisionRhythmMaker(
            ...     denominators=[16],
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
                {
                    \time 7/16
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }
                {
                    \time 3/8
                    {
                        c'16 [
                        c'16
                        c'16
                        c'16
                        c'16
                        c'16 ]
                    }
                }
            }

    Returns sustain mask.
    '''
    from abjad.tools import rhythmmakertools
    indices = list(range(n))
    pattern = patterntools.Pattern(
        indices=indices,
        inverted=inverted,
        )
    mask = rhythmmakertools.SustainMask(pattern=pattern)
    return mask



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_OctavationSpanner_01():
    r'''Octavation has default start set to 1 and stop set to 0.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    spanner =  spannertools.OctavationSpanner()
    attach(spanner, staff[:])

    assert spanner.start == 1
    assert spanner.stop == 0

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \ottava #1
            c'8
            c'8
            c'8
            c'8
            \ottava #0
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_OctavationSpanner_02():

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    spanner = spannertools.OctavationSpanner(start=1)
    attach(spanner, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \ottava #1
            c'8
            cs'8
            d'8
            ef'8
            \ottava #0
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_OctavationSpanner_03():

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    spanner = spannertools.OctavationSpanner(start=1, stop=2)
    attach(spanner, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \ottava #1
            c'8
            cs'8
            d'8
            ef'8
            \ottava #2
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_OctavationSpanner_04():
    r'''Adjacent one-note octavation changes are allowed;
    TODO: check for back-to-back set-octavation at format-
    time and compress to a single set-octavation.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    spanner = spannertools.OctavationSpanner(start=1)
    attach(spanner, staff[0])
    spanner = spannertools.OctavationSpanner(start=2)
    attach(spanner, staff[1])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \ottava #1
            c'8
            \ottava #0
            \ottava #2
            cs'8
            \ottava #0
            d'8
            ef'8
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_OctavationSpanner_05():
    r'''Overlapping octavation spanners are allowed but not well-formed.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    spanner = spannertools.OctavationSpanner(start=1)
    attach(spanner, staff[:4])
    spanner = spannertools.OctavationSpanner(start=2)
    attach(spanner, staff[2:6])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \ottava #1
            c'8
            cs'8
            \ottava #2
            d'8
            ef'8
            \ottava #0
            e'8
            f'8
            \ottava #0
            fs'8
            g'8
        }
        '''
        )

    assert not inspect_(staff).is_well_formed()


# -*- coding: utf-8 -*-
import abjad
import {score_package_name}


class SegmentMaker(abjad.abctools.AbjadObject):
    r'''Segment-maker.
    '''

    ### INITIALIZER ###

    def __init__(self):
        self.score_template = {score_package_name}.tools.ScoreTemplate()

    ### SPECIAL METHODS ###

    def __call__(
        self,
        segment_metadata=None,
        previous_segment_metadata=None,
        ):
        score = self.score_template()
        score['Example Voice'].extend("c'4 ( d'4 e'4 f'4 )")
        lilypond_file = abjad.lilypondfiletools.LilyPondFile.new(
            score,
            includes=['../../stylesheets/stylesheet.ily'],
            )
        return lilypond_file, segment_metadata



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSet___init___01():
    r'''Works with numbers.
    '''

    assert len(pitchtools.PitchSet([12, 14, 18, 19])) == 4


def test_pitchtools_PitchSet___init___02():
    r'''Works with pitches.
    '''

    assert len(pitchtools.PitchSet([NamedPitch(x)
        for x in [12, 14, 18, 19]])) == 4



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__HorizontalBracket_01():

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    bracket = spannertools.HorizontalBracketSpanner()
    attach(bracket, target[:])
    bracket = spannertools.HorizontalBracketSpanner()
    attach(bracket, target[:2])
    bracket = spannertools.HorizontalBracketSpanner()
    attach(bracket, target[2:])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4 \startGroup \startGroup
            c'4 \stopGroup
            c'4 \startGroup
            c'4 \stopGroup \stopGroup
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__HorizontalBracket_02():
    r'''Starting and stopping on the same leaf.
    '''

    string = r'''{ c \startGroup \stopGroup c c c }'''
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__HorizontalBracket_03():
    r'''One group stopping on a leaf, while another begins on the same leaf.
    '''

    string = r'''{ c \startGroup c \stopGroup \startGroup c c \stopGroup }'''
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__HorizontalBracket_04():
    r'''Unterminated.
    '''

    string = r'''{ c \startGroup c c c }'''
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__HorizontalBracket_05():
    r'''Unstarted.
    '''

    string = r'''{ c c c c \stopGroup }'''
    assert pytest.raises(Exception, 'LilyPondParser()(string)')



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import scoretools


def durate_pitch_contour_reservoir(pitch_contour_reservoir):
    r'''Durates pitch contour reservoir.
    '''

    instrument_names = [
        'First Violin',
        'Second Violin',
        'Viola',
        'Cello',
        'Bass',
        ]

    durated_reservoir = {}

    for i, instrument_name in enumerate(instrument_names):
        long_duration = durationtools.Duration(1, 2) * pow(2, i)
        short_duration = long_duration / 2
        rest_duration = long_duration * durationtools.Multiplier(3, 2)

        div = rest_duration // durationtools.Duration(3, 2)
        mod = rest_duration % durationtools.Duration(3, 2)

        initial_rest = scoretools.MultimeasureRest((3, 2)) * div
        if mod:
            initial_rest += scoretools.make_rests(mod)

        durated_contours = [tuple(initial_rest)]

        pitch_contours = pitch_contour_reservoir[instrument_name]
        durations = [long_duration, short_duration]
        counter = 0
        for pitch_contour in pitch_contours:
            contour = []
            for pitch in pitch_contour:
                contour.extend(
                    scoretools.make_leaves([pitch], [durations[counter]])
                    )
                counter = (counter + 1) % 2
            durated_contours.append(tuple(contour))

        durated_reservoir[instrument_name] = tuple(durated_contours)

    return durated_reservoir


# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TaleaRhythmMaker___call___01():

    talea = rhythmmakertools.Talea(
        counts=[-1, 4, -2, 3],
        denominator=16,
        )
    rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=[3, 4],
        )

    divisions = [(2, 8), (5, 8)]
    selections = rhythm_maker(divisions)

    selections = Sequence(selections).flatten()
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(selections)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                \times 4/7 {
                    r16
                    c'4
                    r8
                }
            }
            {
                \time 5/8
                \tweak text #tuplet-number::calc-fraction-text
                \times 5/7 {
                    c'8.
                    r16
                    c'4
                    r8
                    c'8.
                    r16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_TaleaRhythmMaker___call___02():

    talea = rhythmmakertools.Talea(
        counts=[-1, 4, -2, 3],
        denominator=16,
        )
    rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=[3, 4],
        split_divisions_by_counts=[6],
        )

    divisions = [(2, 8), (5, 8)]
    selections = rhythm_maker(divisions)

    selections = Sequence(selections).flatten()
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(selections)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                \times 4/7 {
                    r16
                    c'4
                    r8
                }
            }
            {
                \time 5/8
                {
                    c'8 ~
                }
                \times 2/3 {
                    c'16
                    r16
                    c'4
                    r8
                    c'16 ~
                }
                {
                    c'8
                }
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *
Selection = selectiontools.Selection



def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_01():
    r'''Components that start at the same moment are bad.
    Even if components are all part of the same logical voice.
    '''

    voice = Voice(r'''
        {
            c'8
            d'8
        }
        \new Voice {
            e'8
            f'8
        }
        {
            g'8
            a'8
        }
        ''')

    r'''
    \new Voice {
        {
            c'8
            d'8
        }
        \new Voice {
            e'8
            f'8
        }
        {
            g'8
            a'8
        }
    }
    '''

    assert not Selection._all_are_contiguous_components_in_same_logical_voice(
        [voice, voice[0]])
    assert not Selection._all_are_contiguous_components_in_same_logical_voice(
        voice[0:1] + voice[0][:])
    assert not Selection._all_are_contiguous_components_in_same_logical_voice(
        voice[-1:] + voice[-1][:])


def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_02():
    r'''Is true for strictly contiguous leaves in same staff.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    assert Selection._all_are_contiguous_components_in_same_logical_voice(staff[:])


def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_03():
    r'''Is true for orphan components when allow_orphans is true.
    Is false for orphan components when allow_orphans is False.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    assert Selection._all_are_contiguous_components_in_same_logical_voice(notes)
    assert not Selection._all_are_contiguous_components_in_same_logical_voice(notes, allow_orphans=False)


def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_04():
    r'''Is false for time-reordered leaves in staff.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    assert not Selection._all_are_contiguous_components_in_same_logical_voice(staff[2:] + staff[:2])


def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_05():
    r'''Is true for unincorporated component.
    '''

    assert Selection._all_are_contiguous_components_in_same_logical_voice([Staff("c'8 d'8 e'8 f'8")])


def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_06():
    r'''Is true for empty list.
    '''

    assert Selection._all_are_contiguous_components_in_same_logical_voice([])


def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_07():
    r'''False when components belonging to same logical voice are ommitted.
    '''

    voice = Voice("c'8 d'8 e'8 f'8 g'8 a'8")
    beam = Beam()
    attach(beam, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8
            g'8
            a'8 ]
        }
        '''
        )

    assert not Selection._all_are_contiguous_components_in_same_logical_voice(
        voice[:2] + voice[-2:])


def test_selectiontools_Selection__all_are_contiguous_components_in_same_logical_voice_08():
    r'''False when components belonging to same logical voice are ommitted.
    '''

    voice = Voice(r'''
        {
            c'8 [
            d'8
        }
        {
            e'8
            f'8
        }
        {
            g'8
            a'8 ]
        }
        ''')

    assert not Selection._all_are_contiguous_components_in_same_logical_voice(voice[:1] + voice[-1:])



# -*- coding: utf-8 -*-
import copy
from abjad.tools.datastructuretools.TypedList import TypedList


class TimeSignatureList(TypedList):
    r'''Time signature list.

    ..  container:: example

        Two time signatures:

        ::

            >>> time_signatures = indicatortools.TimeSignatureList(
            ...     [(5, 8), (4, 4)],
            ...     )

        ::

            >>> f(time_signatures)
            indicatortools.TimeSignatureList(
                [
                    indicatortools.TimeSignature((5, 8)),
                    indicatortools.TimeSignature((4, 4)),
                    ]
                )

        ::

            >>> (5, 8) in time_signatures
            True

        ::

            >>> TimeSignature((4, 4)) in time_signatures
            True

        ::

            >>> (3, 4) in time_signatures
            False

        ::

            >>> show(time_signatures) # doctest: +SKIP

        ..  doctest::

            >>> lilypond_file = time_signatures.__illustrate__()
            >>> f(lilypond_file[Score])
            \new Score <<
                \new RhythmicStaff {
                    {
                        \time 5/8
                        s1 * 5/8
                    }
                    {
                        \time 4/4
                        s1 * 1
                    }
                }
            >>

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Collections'

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __illustrate__(self, format_specification=''):
        r'''Formats time signature list.

        ..  container:: example

            ::

                >>> time_signatures = indicatortools.TimeSignatureList(
                ...     [(5, 8), (4, 4)],
                ...     )

            ::

                >>> show(time_signatures) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = time_signatures.__illustrate__()
                >>> f(lilypond_file[Score])
                \new Score <<
                    \new RhythmicStaff {
                        {
                            \time 5/8
                            s1 * 5/8
                        }
                        {
                            \time 4/4
                            s1 * 1
                        }
                    }
                >>

        Returns LilyPond file.
        '''
        import abjad
        measures = abjad.scoretools.make_spacer_skip_measures(self)
        staff = abjad.Staff(measures)
        staff.context_name = 'RhythmicStaff'
        score = abjad.Score([staff])
        lilypond_file = abjad.LilyPondFile.new(score)
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        from abjad.tools import indicatortools
        def coerce(argument):
            if isinstance(argument, tuple):
                return indicatortools.TimeSignature(argument)
            elif isinstance(argument, indicatortools.TimeSignature):
                return copy.copy(argument)
            else:
                message = 'must be pair or time signature: {!r}.'
                message = message.format(argument)
                raise Exception(message)
        return coerce



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHead___cmp___01():

    note_head_1 = scoretools.NoteHead(12)
    note_head_2 = scoretools.NoteHead(12)

    assert not note_head_1 <  note_head_2
    assert      note_head_1 <= note_head_2
    assert      note_head_1 == note_head_2
    assert not note_head_1 != note_head_2
    assert not note_head_1 >  note_head_2
    assert      note_head_1 >= note_head_2


def test_scoretools_NoteHead___cmp___02():

    note_head_1 = scoretools.NoteHead(12)
    note_head_2 = scoretools.NoteHead(13)

    assert not note_head_2 <  note_head_1
    assert not note_head_2 <= note_head_1
    assert not note_head_2 == note_head_1
    assert      note_head_2 != note_head_1
    assert      note_head_2 >  note_head_1
    assert      note_head_2 >= note_head_1


def test_scoretools_NoteHead___cmp___03():

    note_head_1 = scoretools.NoteHead(12)
    note_head_2 = 12

    assert not note_head_1 <  note_head_2
    assert      note_head_1 <= note_head_2
    assert      note_head_1 == note_head_2
    assert not note_head_1 != note_head_2
    assert not note_head_1 >  note_head_2
    assert      note_head_1 >= note_head_2


def test_scoretools_NoteHead___cmp___04():

    note_head_1 = scoretools.NoteHead(12)
    note_head_2 = 13

    assert not note_head_2 <  note_head_1
    assert not note_head_2 <= note_head_1
    assert not note_head_2 == note_head_1
    assert      note_head_2 != note_head_1
    assert      note_head_2 >  note_head_1
    assert      note_head_2 >= note_head_1



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Octave___init___01():
    r'''Initializes octave from empty input.
    '''

    octave = pitchtools.Octave()

    assert octave == pitchtools.Octave(4)



# -*- encoding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools import graphtools


class TestCase(systemtools.TestCase):

    def test_01(self):
        graph_one = graphtools.GraphvizGraph()
        node_one = graphtools.GraphvizNode()
        node_two = graphtools.GraphvizNode()
        node_one.attach(node_two)
        graph_one.extend([node_one, node_two])
        assert str(graph_one) == self.normalize('''
            digraph G {
                node_0;
                node_1;
                node_0 -> node_1;
            }
            ''')
        graph_two = graphtools.GraphvizGraph()
        assert str(graph_two) == self.normalize('''
            digraph G {
            }
            ''')
        graph_two.extend(graph_one)
        assert str(graph_one) == self.normalize('''
            digraph G {
            }
            ''')
        assert str(graph_two) == self.normalize('''
            digraph G {
                node_0;
                node_1;
                node_0 -> node_1;
            }
            ''')
